from django.conf.urls import include, url
from rest_framework import routers

from .app_settings import ACCEPT_INVITE_URL, API_BASE_URL
from .views import InvitationViewSet, accept_invitation

router = routers.SimpleRouter()
router.register(r'{0}'.format(API_BASE_URL), InvitationViewSet)

invitations_patterns = (
    [
        url(
            r'^{0}/{1}/(?P<key>\w+)/?$'.format(
                API_BASE_URL, ACCEPT_INVITE_URL
            ),
            accept_invitation,
            name='accept-invite'
        ),
    ],
    'invitations'
)

urlpatterns = router.urls + [
    url(r'^', include(invitations_patterns)),
]



from django.contrib.auth import get_user_model
from django.core import mail
from django.urls import reverse
from django.utils import timezone
from invitations.utils import get_invitation_model
from rest_framework import status
from rest_framework.test import APITestCase

User = get_user_model()
Invitation = get_invitation_model()


class TestInvitationAPI(APITestCase):

    @classmethod
    def setUpClass(cls):
        super(TestInvitationAPI, cls).setUpClass()
        cls.user = User.objects.create_user(
            email='user@example.org', username='test_user', password='test'
        )

    def test_invitation_send(self):
        self.assertEqual(len(mail.outbox), 0)
        invitation = Invitation.create(email='invited@example.org')
        url = reverse('invitation-send', kwargs={'pk': invitation.pk})
        response = self.client.post(url, format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(len(mail.outbox), 0)
        self.client.login(username=self.user.username, password='test')
        url = reverse('invitation-send', kwargs={'pk': invitation.pk})
        response = self.client.post(url, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(mail.outbox), 1)

    def test_invitation_create_and_send(self):
        self.assertEqual(len(mail.outbox), 0)
        self.assertEqual(Invitation.objects.count(), 0)
        url = reverse('invitation-create-and-send')
        data = {}
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(Invitation.objects.count(), 0)
        self.client.login(username=self.user.username, password='test')
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(Invitation.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)
        data = {'email': 'invite@me.org'}
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(Invitation.objects.count(), 1)

    def test_invitation_send_multiple(self):
        self.assertEqual(len(mail.outbox), 0)
        self.assertEqual(Invitation.objects.count(), 0)
        url = reverse('invitation-send-multiple')
        data = {}
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(Invitation.objects.count(), 0)
        self.client.login(username=self.user.username, password='test')
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(Invitation.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)
        data = {'email': []}
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(len(mail.outbox), 0)
        self.assertEqual(Invitation.objects.count(), 0)
        data = {'email': ['invite@me.org', 'invite2@alt.com']}
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(Invitation.objects.count(), 2)
        data = {'email': ['invite@me.org', 'invite2@alt.com']}
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(Invitation.objects.count(), 2)
        data = {'email': ['invite@me.org']}
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(Invitation.objects.count(), 2)
        data = {'email': ['invite2@alt.com']}
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(Invitation.objects.count(), 2)

    def test_invitation_accept(self):
        invitation = Invitation.create(
            email='invited@email.org',
            inviter=self.user
        )
        invitation_pk = invitation.pk
        invitation.sent = timezone.now()
        invitation.save()
        url = reverse('invitations:accept-invite', kwargs={'key': 'something'})
        data = {}
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_410_GONE)
        self.assertFalse(invitation.accepted)
        url = reverse(
            'invitations:accept-invite',
            kwargs={'key': invitation.key}
        )
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        invitation = Invitation.objects.get(pk=invitation_pk)
        self.assertTrue(invitation.accepted)
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_410_GONE)

    def tearDown(self):
        self.client.logout()



from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _
from invitations.adapters import get_invitations_adapter
from invitations.exceptions import (AlreadyAccepted, AlreadyInvited,
                                    UserRegisteredEmail)
from invitations.utils import get_invitation_model
from rest_framework import serializers

InvitationModel = get_invitation_model()


errors = {
    "already_invited": _("This e-mail address has already been"
                         " invited."),
    "already_accepted": _("This e-mail address has already"
                          " accepted an invite."),
    "email_in_use": _("An active user is using this e-mail address"),
}


class EmailListField(serializers.ListField):
    child = serializers.EmailField()


class InvitationWriteSerializer(serializers.ModelSerializer):
    class Meta:
        model = InvitationModel
        fields = ('email',)

    def _validate_invitation(self, email):
        if InvitationModel.objects.all_valid().filter(
                email__iexact=email, accepted=False):
            raise AlreadyInvited
        elif InvitationModel.objects.filter(
                email__iexact=email, accepted=True):
            raise AlreadyAccepted
        elif get_user_model().objects.filter(email__iexact=email):
            raise UserRegisteredEmail
        else:
            return True

    def validate_email(self, email):
        email = get_invitations_adapter().clean_email(email)

        try:
            self._validate_invitation(email)
        except(AlreadyInvited):
            raise serializers.ValidationError(errors["already_invited"])
        except(AlreadyAccepted):
            raise serializers.ValidationError(errors["already_accepted"])
        except(UserRegisteredEmail):
            raise serializers.ValidationError(errors["email_in_use"])
        return email

    def create(self, validate_data):
        return InvitationModel.create(**validate_data)


class InvitationReadSerializer(serializers.ModelSerializer):
    class Meta:
        model = InvitationModel
        fields = '__all__'


class InvitationBulkWriteSerializer(InvitationWriteSerializer):

    email = EmailListField()

    class Meta(InvitationWriteSerializer.Meta):
        model = InvitationModel
        fields = InvitationWriteSerializer.Meta.fields

    def validate_email(self, email_list):
        if len(email_list) == 0:
            raise serializers.ValidationError(
                _('You must add one or more email addresses')
            )
        for email in email_list:
            email = get_invitations_adapter().clean_email(email)
            try:
                self._validate_invitation(email)
            except(AlreadyInvited):
                raise serializers.ValidationError(errors["already_invited"])
            except(AlreadyAccepted):
                raise serializers.ValidationError(errors["already_accepted"])
            except(UserRegisteredEmail):
                raise serializers.ValidationError(errors["email_in_use"])
            return email_list



from importlib import import_module


def import_callable(path):
    if hasattr(path, '__call__'):
        return path
    else:
        module, attr_name = path.rsplit('.', 1)
        print(module, attr_name)
        return getattr(import_module(module), attr_name)



from django.conf import settings
from invitations.utils import get_invitation_model

from .serializers import \
    InvitationBulkWriteSerializer as DefaultInvitationBulkWriteSerializer
from .serializers import \
    InvitationReadSerializer as DefaultInvitationReadSerializer
from .serializers import \
    InvitationWriteSerializer as DefaultInvitationWriteSerializer
from .utils import import_callable

# Serializers
InvitationReadSerializer = import_callable(
    getattr(
        settings,
        'INVITATION_SERIALIZER_READ',
        DefaultInvitationReadSerializer
    )
)
InvitationWriteSerializer = import_callable(
    getattr(
        settings,
        'INVITATION_SERIALIZER_WRITE',
        DefaultInvitationWriteSerializer
    )
)
InvitationBulkWriteSerializer = import_callable(
    getattr(
        settings,
        'INVITATION_SERIALIZER_WRITE_BULK',
        DefaultInvitationBulkWriteSerializer
        )
)

# Urls
API_BASE_URL = getattr(
    settings, 'INVITATION_API_BASE_URL', 'invitations'
)
ACCEPT_INVITE_URL = getattr(
    settings, 'INVITATION_ACCEPT_INVITE_URL', 'accept-invite'
)
SEND_URL = getattr(
    settings, 'INVITATION_SEND_URL', 'send'
)
CREATE_AND_SEND_URL = getattr(
    settings, 'INVITATION_CREATE_AND_SEND_URL', 'create-and-send'
)
SEND_MULTIPLE_URL = getattr(
    settings, 'INVITATION_SEND_MULTIPLE_URL', 'send-multiple'
)

# Get Invitation model
InvitationModel = get_invitation_model()



from django.contrib import messages
from invitations.adapters import get_invitations_adapter
from invitations.app_settings import app_settings as invitations_settings
from invitations.signals import invite_accepted
from rest_framework import mixins, status, viewsets
from rest_framework.decorators import action, api_view, permission_classes
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.response import Response

from .app_settings import (CREATE_AND_SEND_URL, SEND_MULTIPLE_URL, SEND_URL,
                           InvitationBulkWriteSerializer, InvitationModel,
                           InvitationReadSerializer, InvitationWriteSerializer)


class InvitationViewSet(
    mixins.RetrieveModelMixin,
    mixins.ListModelMixin,
    mixins.CreateModelMixin,
    mixins.DestroyModelMixin,
    viewsets.GenericViewSet
):
    queryset = InvitationModel.objects.all()
    permission_classes = [IsAuthenticated]

    def get_serializer_class(self):
        if self.action in ['list', 'retrieve']:
            return InvitationReadSerializer
        elif self.action == 'send_multiple':
            return InvitationBulkWriteSerializer
        return InvitationWriteSerializer

    def _prepare_and_send(self, invitation, request):
        invitation.inviter = request.user
        invitation.save()
        invitation.send_invitation(request)

    @action(
        detail=True,
        methods=['post'], permission_classes=[IsAuthenticated],
        url_path=SEND_URL
    )
    def send(self, request, pk=None):
        invitation = self.get_object()
        self._prepare_and_send(invitation, request)
        content = {'detail': 'Invite sent'}
        return Response(content, status=status.HTTP_200_OK)

    @action(
        detail=False,
        methods=['post'], permission_classes=[IsAuthenticated],
        url_path=CREATE_AND_SEND_URL
    )
    def create_and_send(self, request):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        email = serializer.data['email']
        invitation = InvitationModel.create(email=email, inviter=request.user)
        self._prepare_and_send(invitation, request)
        content = {'detail': 'Invite sent'}
        return Response(content, status=status.HTTP_200_OK)

    @action(
        detail=False,
        methods=['post'], permission_classes=[IsAuthenticated],
        url_path=SEND_MULTIPLE_URL
    )
    def send_multiple(self, request):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        inviter = request.user
        for email in serializer.data['email']:
            invitation = InvitationModel.create(email=email, inviter=inviter)
            self._prepare_and_send(invitation, request)
        content = {'detail': 'Invite(s) sent'}
        return Response(content, status=status.HTTP_200_OK)


@api_view(('POST', 'GET'))
@permission_classes((AllowAny,))
def accept_invitation(request, key):
    def get_object():
        try:
            return InvitationModel.objects.get(key=key.lower())
        except InvitationModel.DoesNotExist:
            return None

    invitation = get_object()

    login_data = {
        'LOGIN_REDIRECT': invitations_settings.LOGIN_REDIRECT
    }
    signup_data = {
        'SIGNUP_REDIRECT': invitations_settings.SIGNUP_REDIRECT
    }

    if invitations_settings.GONE_ON_ACCEPT_ERROR and \
        (not invitation or
         (invitation and (invitation.accepted or
                          invitation.key_expired()))):
        return Response(status=status.HTTP_410_GONE)

    if not invitation:
        get_invitations_adapter().add_message(
            request,
            messages.ERROR,
            'invitations/messages/invite_invalid.txt'
        )
        return Response(login_data, status=status.HTTP_200_OK)

    if invitation.accepted:
        get_invitations_adapter().add_message(
            request,
            messages.ERROR,
            'invitations/messages/invite_already_accepted.txt',
            {
                'email': invitation.email
            }
        )
        return Response(login_data, status=status.HTTP_200_OK)

    if invitation.key_expired():
        get_invitations_adapter().add_message(
            request,
            messages.ERROR,
            'invitations/messages/invite_expired.txt',
            {
                'email': invitation.email
            }
        )
        return Response(signup_data, status=status.HTTP_200_OK)

    if not invitations_settings.ACCEPT_INVITE_AFTER_SIGNUP:
        invitation.accepted = True
        invitation.save()
        invite_accepted.send(sender=None, email=invitation.email)
        get_invitations_adapter().add_message(
            request,
            messages.SUCCESS,
            'invitations/messages/invite_accepted.txt',
            {
                'email': invitation.email
            }
        )
        signup_data.update(
            {
                'account_verified_email': invitation.email
            }
        )
    return Response(
        signup_data,
        status=status.HTTP_200_OK
    )



from setuptools import setup, find_packages


setup(
    name='django-rest-invitations',
    version='0.1.2',
    author='Marco Federighi',
    author_email='federighi.marco@gmail.com',
    url='http://github.com/fmarco/django-rest-invitations',
    description='Create a set of REST API endpoints to handle invitations',
    packages=find_packages(),
    keywords=['django', 'invitation', 'django-allauth', 'invite', 'rest', 'django-rest-framework', 'drf', 'invitations'],
    zip_safe=False,
    install_requires=[
        'djangorestframework>=3.10',
        'django-invitations==1.9.3'
    ],
    include_package_data=True,
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ],
)